//+------------------------------------------------------------------+
//|                                    FxCraftAllPositinSummary.java |
//|                                         FxCraft :: Jan Jablonski |
//|                                           Copyright 2013 FxCraft |
//|                                 licencja: Creative Commons BY-SA |
//|                                              http://fxcraft.biz/ |
//+------------------------------------------------------------------+
package jforex;

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import com.dukascopy.api.*;

public class FxCraftAllPositionSummary implements IStrategy {
    private IEngine engine;
    private IConsole console;
    private IHistory history;
    private IContext context;
    private IUserInterface userInterface;
    
    private static final String TAB_NAME = "All Position Summary";
    
    private PosSumTableModel1 tableModel1;
    private JTable table1;
    
    public void onStart(IContext context) throws JFException {
        this.engine = context.getEngine();
        this.console = context.getConsole();
        this.history = context.getHistory();
        
        this.context = context;
        this.userInterface = context.getUserInterface();
        placeControlsOnTab(context);
        
    }

    public void onAccount(IAccount account) throws JFException {
    }

    public void onMessage(IMessage message) throws JFException {
    }

    public void onStop() throws JFException {
        userInterface.removeBottomTab(TAB_NAME);
    }

    public void onTick(Instrument instrument, ITick tick) throws JFException {
        try {
            Set<Instrument> tabIn = context.getSubscribedInstruments();
            final ITick tickThis = tick;
            if(tabIn.contains(instrument)){
            SwingUtilities.invokeAndWait(new Runnable() {
                public void run() {
                    double AllPL=0;   
                    Map<Instrument, Vector<IOrder>> tabInOrd = new HashMap<Instrument, Vector<IOrder>>();     
                    try{
                        for (IOrder orderr : engine.getOrders()) {
                            //console.getOut().println("for ins "+ orderr.getInstrument().name());
                            AllPL += orderr.getProfitLossInUSD();
                            if(tabInOrd.containsKey(orderr.getInstrument())){
                                tabInOrd.get(orderr.getInstrument()).add(orderr);
                            }else{
                                tabInOrd.put(orderr.getInstrument(), new Vector<IOrder>());
                                tabInOrd.get(orderr.getInstrument()).add(orderr);
                            }        
                        } 
                        setValuesAll(tabInOrd, AllPL, tickThis);
                    }catch (JFException ex){
                        console.getOut().println(ex);
                    }                   
                }
            });
            }
        } catch (Exception e) {
            console.getOut().println("tu--> "+e);
        }
    }
    
    public void onBar(Instrument instrument, Period period, IBar askBar, IBar bidBar) throws JFException {
    }
    
    private void placeControlsOnTab(IContext context) {
        JPanel mainPanel = userInterface.getBottomTab(TAB_NAME);
        mainPanel.setLayout(new BorderLayout());
        tableModel1 = new PosSumTableModel1();
        table1 = new JTable(tableModel1);
        mainPanel.add(new JScrollPane(table1), BorderLayout.CENTER);
    }
    public void setValuesAll(Map<Instrument, Vector<IOrder>> tab, double all, ITick tick) throws JFException{
        String[][] data = new String[tab.size()+2][5];
        double lpm_currency_realized = 0;
        Date d_from = new Date();
        Date d_to = new Date(tick.getTime());
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy.MM.dd");
        String d_tmp = formatter.format(d_to);
        try{
            d_from = formatter.parse(d_tmp);
        }catch(Exception e){
            console.getErr().println("wyjatek przy parsowaniu: "+e);
        }
        int i = 0;
        for(Map.Entry m : tab.entrySet()){
            Instrument instrumentThis = (Instrument)m.getKey();
            data[i][0] = instrumentThis.name();
            double ll = 0;
            double sh = 0;
            int lln = 0, shn = 0;
            double price = 0;        
            for(IOrder or : (Vector<IOrder>)m.getValue()){
                if(or.isLong()){
                    ll += or.getAmount(); 
                    lln++; 
                }else{
                    sh += or.getAmount();
                    shn++;
                }
                price += or.getProfitLossInUSD();
            }
            data[i][1] = NormalizeDouble(ll) + " / " + NormalizeDouble(sh) + " ("+lln+"/"+shn+")";
            data[i][2] = NormalizeDouble(ll - sh) + "";
            data[i][3] = "";    
            data[i][4] = NormalizeDouble(price) + " USD";
            for(IOrder orderr : history.getOrdersHistory(instrumentThis, d_from.getTime(), d_to.getTime())) {
               lpm_currency_realized += NormalizeDouble((orderr.getProfitLossInPips() * orderr.getAmount())*instrumentThis.getPipValue());
            }
            i++;
        }
        data[i][3] = "Sum:";
        data[i][4] = NormalizeDouble(all) + " USD";
        data[i+1][3] = "Today realized P/L:";
        data[i+1][4] = NormalizeDouble(lpm_currency_realized) + " USD";
        tableModel1.setData(data);
    }
    double NormalizeDouble(double operand) {
           DecimalFormat applydeci = new DecimalFormat("0.00###");
           Double result = new Double(applydeci.format(operand).replace(',', '.')).doubleValue();
           return(result);
    }
}
class PosSumTableModel1 extends AbstractTableModel {
    private String[][] data = new String[0][0];

    public void setData(String[][] data) {
        this.data = data;
        fireTableDataChanged();
    }

    public int getRowCount() {
        return data.length;
    }

    public int getColumnCount() {
        return 5;
    }

    public Object getValueAt(int row, int column) {
            return data[row][column];
    }

    public String getColumnName(int column) {

        switch (column) {
            case 0:
                return "Instrument";
            case 1:
                return "Long/Short amount (number)";
            case 2:
                return "Summary Amount";
            case 3:
                return "Sum";
            case 4:
                return "P/L";
            default:
                return "";
        }
    }
}
